﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Documents.Model;
using System.Windows.Media;
using Telerik.Windows.Documents.Model.Styles;
using Telerik.Windows.Documents.Layout;
using Telerik.Windows.Controls;


namespace CodeBehindDocument
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
        }

        private void richTextBox_Loaded(object sender, RoutedEventArgs e)
        {
            Dispatcher.BeginInvoke(() =>
            {
                this.richTextBox.Document = CreateCodeBehindDocumentWithTable();
            });
        }

        private void getPageNumberOfCurrentDocumentElement_Click(object sender, RoutedEventArgs e)
        {
            DocumentElement element = this.richTextBox.Document.CaretPosition.GetCurrentInlineBox().AssociatedDocumentElement;
            int pageNumber = GetPageNumberForDocumentElement(element);
            MessageBox.Show(pageNumber.ToString());
        }

        private void addMoreContent_Click(object sender, RoutedEventArgs e)
        {
            AddBlocksWithPageBreaksToDocumentInRRTB(this.richTextBox);
        }

        private RadDocument CreateCodeBehindDocumentWithTable()
        {
            RadDocument document = new RadDocument();
            document.LayoutMode = DocumentLayoutMode.Paged;
            document.ParagraphDefaultSpacingAfter = document.ParagraphDefaultSpacingBefore = 0;
            document.PageViewMargin = new SizeF(10, 10);
            document.SectionDefaultPageMargin = new Padding(75, 120, 115, 120);

            Section section = new Section();
            document.Sections.Add(section);

            Table table = CreateTable();
            section.Blocks.Add(table);

            document.ShowFormattingSymbols = true;
            return document;
        }

        private Table CreateTable()
        {
            Table table = new Table(3, 4);
            Color lightGray = Color.FromArgb(20, 30, 30, 80);
            table.Background = lightGray;
            table.Borders.All = new Telerik.Windows.Documents.Model.Border(BorderStyle.Single, Colors.White);

            TableRow[] rows = table.Rows.ToArray();

            TableCell[] cells0 = rows[0].Cells.ToArray();
            Color gray = Color.FromArgb(40, 30, 30, 80);

            foreach (TableCell cell in cells0)
            {
                cell.Background = gray;
                cell.TextAlignment = Telerik.Windows.Documents.Layout.RadTextAlignment.Center;
                cell.VerticalAlignment = Telerik.Windows.Documents.Layout.RadVerticalAlignment.Center;
                cell.Blocks.Add(new Paragraph());
            }

            Span sp00 = new Span("Cell 00");
            sp00.Style = new StyleDefinition();
            sp00.Style.SetPropertyValue(Span.FontWeightProperty, FontWeights.Bold);
            (cells0[0].Blocks.First as Paragraph).Inlines.Add(sp00);

            Span sp01 = new Span("Cell 01");
            sp01.Style = new StyleDefinition();
            sp01.Style.SetPropertyValue(Span.FontWeightProperty, FontWeights.Bold);
            (cells0[1].Blocks.First as Paragraph).Inlines.Add(sp01);

            Span sp02 = new Span("Cell 02");
            sp02.Style = new StyleDefinition();
            sp02.Style.SetPropertyValue(Span.FontWeightProperty, FontWeights.Bold);
            (cells0[2].Blocks.First as Paragraph).Inlines.Add(sp02);

            Span sp03 = new Span("Cell 03");
            sp03.Style = new StyleDefinition();
            sp03.Style.SetPropertyValue(Span.FontWeightProperty, FontWeights.Bold);
            (cells0[3].Blocks.First as Paragraph).Inlines.Add(sp03);

            TableCell[] cells1 = rows[1].Cells.ToArray();

            Paragraph p10 = new Paragraph();
            p10.Inlines.Add(new Span("Cell 10"));
            cells1[0].VerticalAlignment = Telerik.Windows.Documents.Layout.RadVerticalAlignment.Center;
            cells1[0].Blocks.Add(p10);

            Paragraph p111 = new Paragraph();
            p111.Inlines.Add(new Span("Cell 11. First line of text in the cell."));
            cells1[1].Blocks.Add(p111);
            Paragraph p112 = new Paragraph();
            p112.Inlines.Add(new Span("Cell 11. Second line of text in the cell."));
            cells1[1].Blocks.Add(p112);
            Paragraph p113 = new Paragraph();
            p113.Inlines.Add(new Span("Cell 11. Third line of text in the cell."));
            cells1[1].Blocks.Add(p113);
            cells1[1].PreferredWidth = new TableWidthUnit(150f);

            Paragraph p12 = new Paragraph();
            p12.Inlines.Add(new Span("Cell 12 - Aligned bottom right"));
            cells1[2].Blocks.Add(p12);
            cells1[2].TextAlignment = Telerik.Windows.Documents.Layout.RadTextAlignment.Right;
            cells1[2].VerticalAlignment = Telerik.Windows.Documents.Layout.RadVerticalAlignment.Bottom;

            Paragraph p13 = new Paragraph();
            p13.Inlines.Add(new Span("Cell 13 - Aligned right"));
            cells1[3].Blocks.Add(p13);
            cells1[3].TextAlignment = Telerik.Windows.Documents.Layout.RadTextAlignment.Right;

            TableCell[] cells2 = rows[2].Cells.ToArray();

            Paragraph p20 = new Paragraph();
            Span sp20 = new Span("Cell 20 - Bold");
            sp20.Style = new StyleDefinition();
            sp20.Style.SetPropertyValue(Span.FontWeightProperty, FontWeights.Bold);
            p20.Inlines.Add(sp20);
            cells2[0].Blocks.Add(p20);
            cells2[0].ColumnSpan = 3;
            cells2[0].TextAlignment = Telerik.Windows.Documents.Layout.RadTextAlignment.Center;
            cells2[0].VerticalAlignment = RadVerticalAlignment.Bottom;

            Paragraph p23 = new Paragraph();
            p23.Inlines.Add(new Span("Cell 23 - Aligned right"));
            cells2[1].Blocks.Add(p23);
            cells2[1].TextAlignment = Telerik.Windows.Documents.Layout.RadTextAlignment.Right;

            rows[2].Cells.Remove(cells2[2]);
            rows[2].Cells.Remove(cells2[3]);
            return table;
        }

        private void AddBlocksWithPageBreaksToDocumentInRRTB(RadRichTextBox richTextBox)
        {
            RadDocument document = new RadDocument();
            document.Sections.Add(new Section());

            Paragraph paragraphtxt1 = new Paragraph();
            paragraphtxt1.Inlines.Add(new Span("texto texto texto" + FormattingSymbolLayoutBox.PAGE_BREAK));

            Paragraph paragraphtxt2 = new Paragraph();
            paragraphtxt2.Inlines.Add(new Span("texto texto texto" + FormattingSymbolLayoutBox.PAGE_BREAK));

            Paragraph paragraphtxt3 = new Paragraph();
            paragraphtxt3.Inlines.Add(new Span("texto texto texto"));

            document.Sections.First.Blocks.Add(paragraphtxt1);
            document.Sections.First.Blocks.Add(paragraphtxt2);
            document.Sections.First.Blocks.Add(paragraphtxt3);

            MeasureAndArrangeInDefaultSize(document);

            DocumentElement element = this.richTextBox.Document.CaretPosition.GetCurrentInlineBox().AssociatedDocumentElement;
            int pageNumberOfTheDocumentTheCaretIs = GetPageNumberForDocumentElement(element);

            int pageNumber = GetPageNumberForDocumentElement(paragraphtxt3);
            MessageBox.Show(String.Format("The last paragraph in the document which is to be inserted is on page {0} and it is inserted in the new document on page {1}", pageNumber, pageNumber + pageNumberOfTheDocumentTheCaretIs - 1));

            document.Selection.SelectAll();
            this.richTextBox.Document.InsertFragment(document.Selection.CopySelectedDocumentElements());
            this.richTextBox.UpdateEditorLayout();
        }

        public int GetPageNumberForDocumentElement(DocumentElement element)
        {
            LayoutBox layoutBox = element.FirstLayoutBox;
            while (!(layoutBox is SectionLayoutBox))
            {
                layoutBox = layoutBox.Parent;
            }
            return layoutBox.ChildIndex + 1;
        }

        private void MeasureAndArrangeInDefaultSize(RadDocument document)
        {
            document.Measure(RadDocument.MAX_DOCUMENT_SIZE);
            document.Arrange(new RectangleF(PointF.Empty, document.DesiredSize));
        }

    }
}
